/* Copyright 2005 Scott Torborg */

/* This code is released under the GNU General Public License. 
 * For more information, see the COPYING file included in this 
 * package. */

/* diamond.c - A pretty pattern example */

#include "ddf.h"
#include "hsv.c"

#define COLOR_BUFFER 6

inline int
colorAt(unsigned char *mod_data, int y, int x)
{
  int i, c;
  
  x = (x + COLS) % COLS;
  y = (y + ROWS) % ROWS;

  i = (COLS*y + x) * 3;
  c = mod_data[i] << 16;
  c += mod_data[i+1] << 8;
  c += mod_data[i+2];
  return c;
}

inline void
setColor(unsigned char *mod_data, int y, int x, int c)
{
  int i;
  i = (COLS*y + x) * 3;
  mod_data[i] = (c >> 16) & 0xFF;
  mod_data[i+1] = (c >> 8) & 0xFF;
  mod_data[i+2] = c & 0xFF;
}

int
main(int argc, char **argv)
{
  dancefloor* fl;
  int i, j;
  int dist;
  float h, dh;
  char r, g, b;
  unsigned char data[ROWS*COLS*8];
  int colors[COLOR_BUFFER];

  h = 0;
  dh = 0.05;
    
  fl = init();
  
  for (i=0 ; i<COLOR_BUFFER ; i++)
    colors[i] = 0;
  
  for (i=0 ; i<ROWS*COLS*3 ; i++)
    data[i] = 0;

  printf("center at r:%d c:%d\n", ROWS/2, COLS/2);

  while(1) {
 

      hsvrgb(&r, &g, &b, h, 1.0, 0.5);
      colors[0] = (r << 16);
      colors[0] += (g << 8);
      colors[0] += b;

      //advance colors
      for (i=COLOR_BUFFER-1 ; i>0 ; i--)
	colors[i] = colors[i-1];

      for (j=0 ; j<COLS ; j++)
	for (i=0 ; i<ROWS ; i++)
	  {
	    dist = abs(i-ROWS/2)+abs(j-COLS/2);
	    setColor(data, i, j, colors[dist%COLOR_BUFFER]);
	  }
      
      write_dancefloor(fl, data);

      h += dh;
      if (h > 1) h = 0;
      //usleep(80000);

     // cleanup(fd);
    }

  return 0;
}
