/* Copyright 2005 Aaron Sokoloski */

/* This code is released under the GNU General Public License. 
 * For more information, see the COPYING file included in this 
 * package. */

/* fire.c - A demo using the Allegro game library */

#include <allegro.h>
#include <stdlib.h>
#include "ddf.h"

#define FOR_REAL 
#define SCREEN 
const int pix_size = 13;

typedef unsigned char uc;


void render_screen(BITMAP *back) {
  stretch_blit(back, screen,
	       0, 0, back->w, back->h,
	       200, 10, back->w * pix_size, back->h * pix_size);
}

//draw to whatever
void render_floor(dancefloor *fl, BITMAP *bitmap) {
  static unsigned char b[COLS*ROWS*3];

  // Only write to the floor if we have to
#ifdef FOR_REAL
  
  int x, y, col, i = 0;

  for (y = 0; y < ROWS; y++) {
    for (x = 0; x < COLS; x++) {
      col = getpixel(bitmap, x, y);
      b[i]     = (unsigned char)getr24(col);
      b[i + 1] = (unsigned char)getg24(col);
      b[i + 2] = (unsigned char)getb24(col);
      i += 3;
    }
  }
  write_dancefloor(fl, b);
#endif
#ifdef SCREEN
  render_screen(bitmap);
#endif
}

int main(void) {
  dancefloor *fl;

  allegro_init();  // need these functions so we can draw coolness to a buffer, then to the floor
  set_color_depth(24);

#ifdef FOR_REAL
  fl = init();
#endif
  set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0);
  install_keyboard();

  static BITMAP *buf = NULL;
  if (buf == NULL)   buf = create_bitmap(COLS, ROWS);
  
  rectfill(buf, 0, 0, buf->w, buf->h, makecol(100, 100, 100));

  int i = 0;
  int x, y;

  int fire_buffer[COLS + 1][ROWS];
  int colors[256];

  for (i = 0; i < 64; i++) colors[i] = makecol(i * 4, 0, 0);
  for (i = 0; i < 64; i++) colors[i + 64] = makecol(255, i * 4, 0);
  for (i = 0; i < 64; i++) colors[i + 128] = makecol(255, 255, i*4);
  for (i = 0; i < 64; i++) colors[i + 192] = makecol(255 - i*4, 255 - i*4, 255);

  while (!keypressed()) {
    for (y = 0; y < ROWS; y++) {
      fire_buffer[COLS][y] = (rand() % 2) * 255;
    }
    
    for (x = 0; x < COLS; x++) {
      for (y = 0; y < ROWS; y++) {
	int col = 0;

	col = fire_buffer[x][y];
	
	for (i = y - 1; i <= y + 1; i++) {
	  col += fire_buffer[x+1][(i + ROWS) % (ROWS)];
	}
	col = col / 4;
	col -= 7;

	if (col < 0) col = 0;
	if (col > 255) col = 255;
	fire_buffer[x][y] = col;
	
	putpixel(buf, x, y, colors[col]);
      }
    }

    render_floor(fl, buf);
  }

  allegro_exit();
#ifdef FOR_REAL
  free(fl);
#endif

  return 0;
}

END_OF_MAIN()

