/* Copyright 2005 Grant Elliott */

/* This code is released under the GNU General Public License. 
 * For more information, see the COPYING file included in this 
 * package. */

/* DISCO DANCE FLOOR XMMS BEAT DETECTION
 * First East, MIT
 * April 2005 */
 
#include <xmms/plugin.h>
#include "beat.h"


#define EHISTORY 44
#define ZTHRESH 1.5

//The energy buffer remembers the last 44 calls (circular)
//try to be at least a little bit consistant about
//calling get_beat...
static int es[64][EHISTORY];
//Index to where to put the current energy vector
static int eindex=0;
//the number of good points in the buffer
static int egood=0;
//running avg and var of each band
static float eavg[64];
static float evar[64];
//there's a beat right now
static int beatflag;
//primary band
static int cband=0;

//calls since last beat
static int calls=0;
//calls between last set of beats
static int callavg=0;

//Test beat detection by blinking one led
void beat_test(gint16 (*shared_freq_data)[256],unsigned char* floor_data)
{
  int b;
	//printf("Entering beat_test\n");
	b=get_beat(shared_freq_data);
	if (b==1)
	{
		floor_data[0]=100;
	} else {
		floor_data[0]=0;
	}
}

//Call reset_beat at the start of every song or
//very bad (confusing) things will happen)
void reset_beat()
{
	printf("Resetting Beat Detection\n");
	memset(es,0,sizeof(es));
	memset(eavg,0,sizeof(eavg));
	memset(evar,0,sizeof(evar));
	egood=0;
	eindex=0;
	beatflag=0;
	calls=0;
	callavg=0;
}
	
// take shared frequency data and return 1
// if there is a beat, 0 otherwise
int get_beat(gint16 (*shared_freq_data)[256])
{
	//printf("Entering get_beat\n");
	int i,j,cnt,beat=0;
	long temp,tempsum; 
	float tempvar,timebon,zs[64];
	printf("cband=%d, eavg=%f, estd=%f\n",cband,eavg[cband],evar[cband]);
	for (i=0;i<63;i++) //increment through bands
	{
		//determine the energy of this band
		temp=shared_freq_data[0][2*i]*shared_freq_data[0][2*i]
			+shared_freq_data[1][2*i]*shared_freq_data[1][2*i]
			+shared_freq_data[0][2*i+1]*shared_freq_data[0][2*i+1]
			+shared_freq_data[1][2*i+1]*shared_freq_data[1][2*i+1];
        zs[i]=((float) temp-eavg[i])/evar[i];
		if (cband!=i)
		{
            zs[i]/=60;
        }
        if (cband==i-1 || cband==i+1)
        {
            zs[i]*=30;
		}
		//update the avg and var of this band
		j=eindex;
		cnt=0;
		tempvar=0;
		tempsum=0;
		while(cnt<egood)
		{
			tempvar+=((float) es[i][j]-eavg[i])*((float) es[i][j]-eavg[i]); //cheating... using old eavg
			tempsum+=es[i][j];
			cnt++;
			j--;
			if (j<0)
			{
			  j=EHISTORY-1;
			}
		}
		evar[i]=sqrt(tempvar/(egood-1));
		eavg[i]=((float) tempsum)/(egood);
		//update the energy history
		es[i][eindex]=temp;
	}
	if (abs(calls-callavg)<=8)
	{
	   timebon=ZTHRESH/(2.5-abs(calls-callavg)/8);
	} else {
	   timebon=ZTHRESH;
	}
	if (zs[cband]>timebon)
	{
	   beat=1;
    } else if (beatflag==0) {
    	for (i=0;i<63;i++)
    	{
    	   if (zs[i]>zs[cband] && zs[i]>timebon)
    	   {
    	       cband=i;
    	   }
    	}
    }
	//buffer bookkeeping
	if (egood<EHISTORY)
	{
		egood=egood+1;
	}
	eindex++;
	if (eindex>EHISTORY)
	{
	   eindex=0;
	}
	if (beatflag==1)
	{
		beat=0;
	}
	if (beat)
    {
        callavg=calls;
        calls=0;
    } else {
        calls++;
    }
	return beat;
}