/* Copyright 2005 Grant Elliott */

/* This code is released under the GNU General Public License. 
 * For more information, see the COPYING file included in this 
 * package. */

/* DISCO DANCE FLOOR XMMS PLUGIN
 * First East, MIT
 * March 2005 */
 
#include <stdio.h>
#include <stdlib.h>
#include <xmms/plugin.h>
#include <xmms/xmmsctrl.h>
#include <string.h>
#include <sys/malloc.h>
#include <time.h>
#include <libgen.h>
#include <gtk/gtk.h>
#include <glib.h>

#include "ddf.h"

#include "spectrum.h"
#include "beat.h"
#include "vis.h"

/* about window parameters */
#define ABOUT_MARGIN 10
#define ABOUT_WIDTH 300
#define ABOUT_HEIGHT 150

/* config window parameters */
#define CONF_MARGIN 10
#define CONF_WIDTH 600
#define CONF_HEIGHT 500

/* default animation filenames */
#define DEFAULT_ANIM1 "/Users/storborg/DevProjects/disco/anim/checker.ddf"
#define DEFAULT_ANIM2 "/Users/storborg/DevProjects/disco/anim/rings5.ddf"
#define DEFAULT_FOLDER "/Users/storborg/DevProjects/disco/anim/"

/* default animation speed, from 0 to 100 */
#define DEFAULT_ANIM_SPEED 80.0

/* GTK globals */

/* xmms main window */
extern GtkWidget *mainwin;
/* xmms dockwinlist */
extern GList *dock_window_list;

static GtkWidget *win_about = NULL;
static GtkWidget *win_conf = NULL;

/* forward declarations */
static void ddf_init(void);
static void ddf_cleanup(void);
static void ddf_about(void);
static void ddf_configure(void);
static void ddf_playback_start(void);
static void ddf_playback_stop(void);
static void ddf_render_freq(gint16 freq_data[2][256]);

/* plugin callback functions and information */
VisPlugin ddf_vtable = {
	/* handle, filled in by xmms */
	0,
	/* filename, filled in by xmms */
	0,
	/* session ID */
	0,
	/* plugin description, shows up in config box */
	"Disco Dance Floor",
	
	/* # of PCM channels for render_pcm() */
	0,
	/* # of freq channels wanted for render_freq() */
	2,
	
	/* called when plugin is enabled, or at xmms launch */
	ddf_init,
	/* called when plugin is disabled, or at xmms exit */
	ddf_cleanup,
	/* show the about box */
	ddf_about,
	/* show the configure box */
	ddf_configure,
	/* called to disable plugin, filled in by xmms */
	0,
	/* called when playback starts */
	ddf_playback_start,
	/* called when playback stops */
	ddf_playback_stop,
	/* render the PCM data, must return quickly */
	0,
	/* render the frequency data, must return quickly */
	ddf_render_freq
};

/* XMMS entry point */
VisPlugin
*get_vplugin_info(void)
{
	/* return plugin info */
	return &ddf_vtable;
}

/* file loader select */
int file_select;

/* animation 1 filename */
//char* anim1_filename;
char anim1_filename[255];
/* animation 1 data */
unsigned char anim1_data[1000][ROWS*COLS*3];
/* number of animation frames in animation 1*/
int anim1_num_frames;

/* animation 2 filename */
//char* anim2_filename;
char anim2_filename[255];
/* animation 2 data */
unsigned char anim2_data[1000][ROWS*COLS*3];
/* number of animation frames in animation 2*/
int anim2_num_frames;

/* global dance floor struct */
dancefloor* fl;

/* thread control commands */
enum { GO, STOP } thread_control;

/* shared frequency data */
gint16 shared_freq_data[2][256];
/* mutex for shared frequency data */
pthread_mutex_t* mutex;

/* mutex for display config selector */
pthread_mutex_t* config_mutex;
/* display config selector */
int anim_select;
/* animation speed selector */
int anim_speed;

int
read_id3(char* fname)
{
    FILE *mp3file;
    unsigned long rsize;
    char tmp[130], *ptr;
    char comment[31];
    
    char *t;
    char *ddf_fname;
    
    /* try to open the mp3 file for reading */
    if((mp3file = fopen(fname,"rb")) == NULL) {
        fprintf(stderr,"Error! Failed to open file for reading ID3 tags.\n");
        return 1;
    }
    
    /* go to 128 bytes before the end of the file */
    fseek(mp3file, -128l, SEEK_END);
    
    /* read the last 128 bytes into tmp */
    rsize = fread(&tmp, 1, 128, mp3file);
    /* check to see how many bytes were actually read */
    if(rsize < 128) {
        fprintf(stderr,"Error! Only read %d bytes of the ID3 tag area.\n",rsize);
        return 2;
    }
    ptr = tmp;

    /* check to see if the first 3 bytes are actually an ID3 tag */
    if(strncmp(ptr,"TAG",3) != 0) {
        fprintf(stderr,"Error! No ID3 tags found!\n");
        return 3;
    }

    /* index to comment field */
    ptr += 97;
    
    /* read the comment field */
    strncpy(comment, ptr, 30);
    printf("comment is [%s]\n",comment);
    
    /* here we need to parse the comment field
     * and set anim1_filename and anim_speed */
    t = strsep(&ptr,":");
    if(strcmp(t,"DDF")) {
        /* not a valid DDF field */
        return 4;
    } else {
        /* got a DDF comment */
        t = strsep(&ptr, ":");
        ddf_fname = t;
        
        printf("ID3: got filename %s\n",ddf_fname);
        
        /* make filename */
        strcpy(anim1_filename,DEFAULT_FOLDER);
        strcat(anim1_filename,ddf_fname);

        /* get animation speed */
        t = strsep(&ptr, ":");
        anim_speed = atoi(t);
        printf("ID3: Set anim_speed to %d\n",anim_speed);
    }

    /* if we make it this far, return 0 to indicate no errors */
    return 0;
    
    fclose(mp3file);
}

/* plugin thread loops here */
void*
worker_func(void* foo)
{
	
	unsigned char *floor_data = malloc(ROWS*COLS*3);
	int myanim, frame;
	gchar *currentsong;
	gchar *lastsong;
	
	currentsong = "";
	lastsong = "";
	frame = 0;
	
	while(thread_control == GO) {
		/* reset floor data */
		memset(floor_data, 0, ROWS*COLS*3);
		
		/* lock the config selector and copy it */
		pthread_mutex_lock(config_mutex);
		myanim = anim_select;
		pthread_mutex_unlock(config_mutex);
		
		if(xmms_remote_is_playing(ddf_vtable.xmms_session)) {
			/* playing, get current song */
			currentsong = xmms_remote_get_playlist_file(ddf_vtable.xmms_session,xmms_remote_get_playlist_pos(ddf_vtable.xmms_session));
			if(strcmp(currentsong,lastsong) != 0) {
				/* song has been updated, update file! */
				printf("new song! %s\n",currentsong);
				lastsong = currentsong;
				
				reset_beat();
				
				/* doing ID3 detection */
				if(myanim == 0) {
					/* read ID3 tag and set anim1_filename and anim_speed */
					read_id3(currentsong);
					load_animation_file(1);
				}
			}
			/* otherwise ignore it */
		}
		
		
		switch(myanim) {
			case 0:
				/* note that there is NO break here */
			case 1:
				/* do the checkers animation normally */
				memcpy(floor_data,anim1_data[frame],ROWS*COLS*3);
				frame++;
				if(frame > anim1_num_frames) {
					/* wrap animation */
					frame = 0;
				}
				break;
			case 2:
				/* do the checkers animation normally */
				memcpy(floor_data,anim2_data[frame],ROWS*COLS*3);
				frame++;
				if(frame > anim2_num_frames) {
					/* wrap animation */
					frame = 0;
				}
				break;
			case 3:
				/* lock the frequency data mutex */
				pthread_mutex_lock(mutex);
				/* render vis */
				render_vis(shared_freq_data, floor_data);
				/* unlock the frequency data mutex */
				pthread_mutex_unlock(mutex);
				break;
			case 4:
				/* lock the frequency data mutex */
				pthread_mutex_lock(mutex);
				/* render spectrum analyzer */
				render_spectrum(shared_freq_data, floor_data);
				/* unlock the frequency data mutex */
				pthread_mutex_unlock(mutex);
				break;
			case 6:
				/* black */
				break;
		}
		
		/* wait for a delay corresponding to inverse of
		 * animation speed */
		usleep((100.0-anim_speed)*10000);
		
		/* rock the house! */		
		write_dancefloor(fl, floor_data);
	}
	
	free(floor_data);
	
	printf("DDF: worker thread exiting\n");
}

/* load an animation file into the animation 1 buffer */
int
load_animation_file(int num)
{
	FILE* file;
	switch(num) {
		case 1:
			anim1_num_frames = 0;
			/* open animation file */
			if((file = fopen(anim1_filename,"rb")) == NULL) {
				printf("File not found!\n");
				return;
			}	
			/* read the animation */
			while(!feof(file) && anim1_num_frames < 1000) {
				fread(anim1_data[anim1_num_frames],1,ROWS*COLS*3,file);
				anim1_num_frames++;
			}
			anim1_num_frames -= 2;
			fclose(file);
			printf("Buffer 1 loaded %d frames : %s\n",anim1_num_frames+1,anim1_filename);
			break;
		case 2:
			anim2_num_frames = 0;
			/* open animation file */
			if((file = fopen(anim2_filename,"rb")) == NULL) {
				printf("File not found!\n");
				return;
			}	
			/* read the animation */
			while(!feof(file) && anim2_num_frames < 1000) {
				fread(anim2_data[anim2_num_frames],1,ROWS*COLS*3,file);
				anim2_num_frames++;
			}
			anim2_num_frames -= 2;
			fclose(file);
			printf("Buffer 2 loaded %d frames : %s\n",anim2_num_frames+1,anim2_filename);
			break;
	}
}

/* called when plugin is enabled, or at xmms launch */
static void
ddf_init(void) 
{
	printf("DDF: Initializing\n");
	
	strcpy(anim1_filename,DEFAULT_ANIM1);
	strcpy(anim2_filename,DEFAULT_ANIM2);
	anim_speed = DEFAULT_ANIM_SPEED;
	
	load_animation_file(1);
	load_animation_file(2);
	
	/* initialize floor */
	fl = init();
	
	/* make the worker thread go once it's ready */
	thread_control = GO;
	
	/* initialize frequency data mutex */
	mutex = (pthread_mutex_t *)malloc(sizeof(pthread_mutex_t));
	pthread_mutex_init(mutex, NULL);
	
	/* initialize config data mutex */
	config_mutex = (pthread_mutex_t *)malloc(sizeof(pthread_mutex_t));
	pthread_mutex_init(config_mutex, NULL);
	
	/* reset shared frequency data to zeroes */
	memset(shared_freq_data, 0, sizeof(gint16) * 2 * 256);

	pthread_mutex_lock(config_mutex);
	/* default behavior is to use ID3 tag data */
	anim_select = 0;
	pthread_mutex_unlock(config_mutex);

	/* spawn worker thread */
	pthread_t worker;
	pthread_create(&worker, NULL, worker_func, NULL);
}

/* called when plugin is disabled, or at xmms exit */
static void
ddf_cleanup(void)
{
	printf("DDF: Cleaning up\n");
	/* only do this if we actually started to begin with */
	powerdown_dancefloor(fl);
	
	if(thread_control == GO) {
		/* stop dance floor */
		thread_control = STOP;
		/* cleanup dance floor */
		cleanup(fl);
	}
	
	printf("DDF: Cleanup completed\n");
}

/* callback to close about window */
static void
on_button_about_close_clicked(GtkButton *button, gpointer user_data)
{
	gtk_widget_destroy(win_about);
	win_about = NULL;
}

/* show the about box */
static void
ddf_about(void)
{
	/* declare GTK shit */
	GtkWidget *vb_main;
	GtkWidget *frame;
	GtkWidget *label_author;
	GtkWidget *button_about_close;
	
	/* if about window is already showing,
	 * just return */
	if(win_about) {
		return;
	}
		
	/* create about window */
	win_about = gtk_window_new (GTK_WINDOW_DIALOG);
	gtk_widget_realize(win_about);
	/* set about window title */
	gtk_window_set_title(GTK_WINDOW(win_about), "About");
	/* connect callback for close button */
	gtk_signal_connect(GTK_OBJECT(win_about), "destroy",
			GTK_SIGNAL_FUNC(gtk_widget_destroyed),
			&win_about);
	/* set window border */
	gtk_container_set_border_width(GTK_CONTAINER(win_about), ABOUT_MARGIN);
	
	/* add box for shit */
	vb_main = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(win_about), vb_main);
	gtk_widget_show(vb_main);
	
	/* add frame widget */
	frame = gtk_frame_new("Disco Dance Floor");
	gtk_box_pack_start(GTK_BOX(vb_main), frame, TRUE, TRUE, 0);
	gtk_widget_set_usize(frame, ABOUT_WIDTH - ABOUT_MARGIN*2,
			ABOUT_HEIGHT - ABOUT_MARGIN*2);
	gtk_container_set_border_width(GTK_CONTAINER(frame), ABOUT_MARGIN);
	gtk_widget_show(frame);
	
	/* ego label */
	label_author = gtk_label_new("Visualization plugin for XMMS\n"
															"Written by Scott Torborg\n"
															"storborg@mit.edu");
	gtk_container_add(GTK_CONTAINER(frame), label_author);
	gtk_widget_show(label_author);
	
	/* add another close button */
	button_about_close = gtk_button_new_with_label("Close");
	gtk_box_pack_start(GTK_BOX(vb_main), button_about_close, FALSE, FALSE, 0);
	gtk_widget_show(button_about_close);
	/* connect that to the a close handler */
	gtk_signal_connect(GTK_OBJECT(button_about_close), "clicked",
			GTK_SIGNAL_FUNC(on_button_about_close_clicked),
			GTK_OBJECT(win_about));
	/* show the window */
	gtk_widget_show(win_about);
	
	return;
}

/* callback to close conf window */
static void
on_button_conf_close_clicked(GtkButton *button, gpointer user_data)
{
	gtk_widget_destroy(win_conf);
	win_conf = NULL;
}

/* selector radio callback */
static void
selector_radio_conf_clicked(GtkWidget *widget, gpointer data)
{
	if(GTK_TOGGLE_BUTTON(widget)->active) {
		/* this is the active radio callback */
		pthread_mutex_lock(config_mutex);
		anim_select = (int) data;
		pthread_mutex_unlock(config_mutex);
	} else {
		/* ignore these messages for now */
	}
}

/* file selection done callback, data is the file selection
 * widget */
void
conf_file_ok_sel(GtkWidget* w, GtkWidget* fs)
{
	char* fname;
	fname = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
	/* select the right filename */
	switch(file_select) {
		case 1:
			printf("resetting anim1_filename to %s\n",fname);
			strcpy(anim1_filename, fname);
			break;
		case 2:
			printf("resetting anim2_filename to %s\n",fname);
			strcpy(anim2_filename, fname);
			break;
	}
	/* fill the animation buffer */
	load_animation_file(file_select);	
	/* close window once we're done */
	gtk_widget_destroy(fs);
}

/* file select callback, data selects which animation buffer
 * to load file into */
static void
file_select_conf_clicked(GtkWidget* widget, gpointer data)
{
	GtkWidget* filew;
	
	/* select buffer */
	file_select = (int) data;
	
	/* spawn file selector box */
	printf("button %d pressed!\n",(int) data);
	
	/* make new file selection widget */
	filew = gtk_file_selection_new("Select DDF File");
	
	/* connect close button to destroy window */
	gtk_signal_connect(GTK_OBJECT(filew), "destroy",
			GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(filew));
	
	/* connect the ok button to conf_file_ok_sel function */
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filew)->ok_button),
			"clicked", GTK_SIGNAL_FUNC(conf_file_ok_sel), filew);
	
	/* connect the cancel button to destroy window */
	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION
			(filew)->cancel_button), "clicked",
			GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(filew));
	
	/* show the widget */
	gtk_widget_show(filew);
	
	return;
}

/* reinitialize button clicked */
void
reinit_conf_clicked(GtkWidget* widget, gpointer data)
{
	/* call reinitialize command */
	reinit_dancefloor(fl);
}

/* enable button clicked */
void
enable_conf_clicked(GtkWidget* widget, gpointer data)
{
	if(GTK_TOGGLE_BUTTON(widget)->active) {
		/* toggle button is down now */
		
	} else {
		/* toggle button is up now */
		
	}
}

/* animation speed slider changed */
void
speed_conf_changed(GtkAdjustment *adj, gpointer data)
{
	pthread_mutex_lock(config_mutex);
	anim_speed = adj->value;
	pthread_mutex_unlock(config_mutex);
}

/* show the configure box */
static void
ddf_configure(void)
{
	printf("DDF: Configure\n");

	/* declare GTK shit */
	GtkWidget* vb_main;
	GtkWidget* selector_radio;
	GtkWidget* button_conf_close;
	GtkWidget* frame;
	GtkWidget* file_select_button;
	GtkWidget* options_button;
	GtkWidget* animtable;
	GtkWidget* optionstable;
	GtkWidget* options_scale;
	GtkObject* speed_adjust;
	GtkWidget* label;
	GSList *group;
	int myanim;
	
	/* if conf window is already showing,
	 * just return */
	if(win_conf) {
		return;
	}
	
	/* copy animation selector */
	pthread_mutex_lock(config_mutex);
	myanim = anim_select;
	pthread_mutex_unlock(config_mutex);
	
	/* create conf window */
	win_conf = gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_widget_realize(win_conf);
	/* set conf window title */
	gtk_window_set_title(GTK_WINDOW(win_conf), "Disco Dance Floor Configuration");
	/* connect callback for close button */
	gtk_signal_connect(GTK_OBJECT(win_conf), "destroy",
			GTK_SIGNAL_FUNC(gtk_widget_destroyed),
			&win_conf);
	/* set window border */
	gtk_container_set_border_width(GTK_CONTAINER(win_conf), CONF_MARGIN);
	
	/* add box for shit */
	vb_main = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(win_conf), vb_main);
	gtk_widget_show(vb_main);
	
	/* add frame widget */
	frame = gtk_frame_new("Animation");
	gtk_box_pack_start(GTK_BOX(vb_main), frame, TRUE, TRUE, 0);
	gtk_widget_set_usize(frame, CONF_WIDTH - CONF_MARGIN*2,
			CONF_HEIGHT/2 - CONF_MARGIN*2);
	gtk_container_set_border_width(GTK_CONTAINER(frame), CONF_MARGIN);
	gtk_widget_show(frame);
	
	/* make table for radio buttons and file load buttons */
	animtable = gtk_table_new(6, 2, TRUE);
	gtk_container_add(GTK_CONTAINER(frame), animtable);
	
	/* add another frame widget */
	frame = gtk_frame_new("Options");
	gtk_box_pack_start(GTK_BOX(vb_main), frame, TRUE, TRUE, 0);
	gtk_widget_set_usize(frame, CONF_WIDTH - CONF_MARGIN*2,
			CONF_HEIGHT/2 - CONF_MARGIN*2);
	gtk_container_set_border_width(GTK_CONTAINER(frame), CONF_MARGIN);
	gtk_widget_show(frame);

	/* make table for options sliders and checkboxes */
	optionstable = gtk_table_new(6, 2, TRUE);
	gtk_container_add(GTK_CONTAINER(frame), optionstable);
	
	/* autoselect radio button */
	selector_radio = gtk_radio_button_new_with_label (NULL, "Autoselect based on ID3 Tag");
	if(myanim == 0) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (selector_radio), TRUE);
	}
	gtk_table_attach_defaults(GTK_TABLE(animtable),selector_radio,0,1,0,1);
	gtk_widget_show (selector_radio);
	gtk_signal_connect(GTK_OBJECT(selector_radio), "clicked",
			GTK_SIGNAL_FUNC(selector_radio_conf_clicked),
			(gpointer) 0);
		
	/* animation 1 radio button */
	group = gtk_radio_button_group (GTK_RADIO_BUTTON (selector_radio));
	selector_radio = gtk_radio_button_new_with_label(group, 
			"Prerendered DDF animation 1");
	if(myanim == 1) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (selector_radio), TRUE);
	}
	gtk_table_attach_defaults(GTK_TABLE(animtable),selector_radio,0,1,1,2);
	gtk_widget_show (selector_radio);
	gtk_signal_connect(GTK_OBJECT(selector_radio), "clicked",
			GTK_SIGNAL_FUNC(selector_radio_conf_clicked),
			(gpointer) 1);
	
	/* animation 1 file select button */
	file_select_button = gtk_button_new_with_label(basename(anim1_filename));
	gtk_table_attach(GTK_TABLE(animtable),file_select_button,1,2,1,2,
			GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 10, 5);
	gtk_widget_show(file_select_button);
	gtk_signal_connect(GTK_OBJECT(file_select_button), "clicked",
			GTK_SIGNAL_FUNC(file_select_conf_clicked),
			(gpointer) 1);
	
	/* animation 2 radio button */
	selector_radio = gtk_radio_button_new_with_label(
			gtk_radio_button_group (GTK_RADIO_BUTTON (selector_radio)),
			"Prerendered DDF animation 2");
	if(myanim == 2) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (selector_radio), TRUE);
	}
	gtk_table_attach_defaults(GTK_TABLE(animtable),selector_radio,0,1,2,3);
	gtk_widget_show (selector_radio);
	gtk_signal_connect(GTK_OBJECT(selector_radio), "clicked",
			GTK_SIGNAL_FUNC(selector_radio_conf_clicked),
			(gpointer) 2);
	
	/* animation 2 file select button */
	file_select_button = gtk_button_new_with_label(basename(anim2_filename));
	gtk_table_attach(GTK_TABLE(animtable),file_select_button,1,2,2,3,
			GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 10, 5);
	gtk_widget_show(file_select_button);
	gtk_signal_connect(GTK_OBJECT(file_select_button), "clicked",
			GTK_SIGNAL_FUNC(file_select_conf_clicked),
			(gpointer) 2);
	
	/* random radio button */
	selector_radio = gtk_radio_button_new_with_label(
			gtk_radio_button_group (GTK_RADIO_BUTTON (selector_radio)),
			"Visualization");
	if(myanim == 3) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (selector_radio), TRUE);
	}
	gtk_table_attach_defaults(GTK_TABLE(animtable),selector_radio,0,1,3,4);
	gtk_widget_show (selector_radio);
	gtk_signal_connect(GTK_OBJECT(selector_radio), "clicked",
			GTK_SIGNAL_FUNC(selector_radio_conf_clicked),
			(gpointer) 3);
	
	/* spectrum analyzer radio button */
	selector_radio = gtk_radio_button_new_with_label(
			gtk_radio_button_group (GTK_RADIO_BUTTON (selector_radio)),
			"FFT spectrum analzer");
	if(myanim == 4) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (selector_radio), TRUE);
	}
	gtk_table_attach_defaults(GTK_TABLE(animtable),selector_radio,0,1,4,5);
	gtk_widget_show (selector_radio);
	gtk_signal_connect(GTK_OBJECT(selector_radio), "clicked",
			GTK_SIGNAL_FUNC(selector_radio_conf_clicked),
			(gpointer) 4);

	/* black radio button */
	selector_radio = gtk_radio_button_new_with_label(
			gtk_radio_button_group (GTK_RADIO_BUTTON (selector_radio)),
			"Black");
	if(myanim == 10) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (selector_radio), TRUE);
	}
	gtk_table_attach_defaults(GTK_TABLE(animtable),selector_radio,0,1,5,6);
	gtk_widget_show (selector_radio);
	gtk_signal_connect(GTK_OBJECT(selector_radio), "clicked",
			GTK_SIGNAL_FUNC(selector_radio_conf_clicked),
			(gpointer) 10);
	
	/* show the top table */
	gtk_widget_show(animtable);
	
	/* make animation speed adjustment */
	speed_adjust = gtk_adjustment_new(anim_speed, 0, 101.0, 1.0, 1.0, 1.0); 
	gtk_signal_connect(GTK_OBJECT(speed_adjust), "value_changed",
			GTK_SIGNAL_FUNC(speed_conf_changed), NULL);
	
	/* animation speed slider */
	options_scale = gtk_hscale_new(GTK_ADJUSTMENT(speed_adjust));
	gtk_table_attach(GTK_TABLE(optionstable),options_scale,0,1,0,1,
			GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 10, 5);
	gtk_widget_show(options_scale);
	
	/* make animation speed label */
	label = gtk_label_new("Animation Speed Adjustment");
	gtk_table_attach_defaults(GTK_TABLE(optionstable),label,1,2,0,1);
	gtk_widget_show(label);
	
	/* reinit button */
	options_button = gtk_button_new_with_label("Reinitialize Floor");
	gtk_table_attach(GTK_TABLE(optionstable),options_button,0,1,5,6,
			GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 10, 5);
	gtk_widget_show(options_button);
	gtk_signal_connect(GTK_OBJECT(options_button), "clicked",
			GTK_SIGNAL_FUNC(reinit_conf_clicked), NULL);
	
	/* enable button */
	options_button = gtk_toggle_button_new_with_label("Enable Floor");
	gtk_table_attach(GTK_TABLE(optionstable),options_button,1,2,5,6,
			GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 10, 5);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(options_button), TRUE);
	gtk_widget_show(options_button);
	gtk_signal_connect(GTK_OBJECT(options_button), "clicked",
			GTK_SIGNAL_FUNC(enable_conf_clicked), NULL);
	
	/* show the bottom table */
	gtk_widget_show(optionstable);
	
	/* add another close button */
	button_conf_close = gtk_button_new_with_label("Close");
	gtk_box_pack_start(GTK_BOX(vb_main), button_conf_close, FALSE, FALSE, 0);
	gtk_widget_show(button_conf_close);
	
	/* connect that to the close handler */
	gtk_signal_connect(GTK_OBJECT(button_conf_close), "clicked",
			GTK_SIGNAL_FUNC(on_button_conf_close_clicked),
			GTK_OBJECT(win_conf));
	
	/* show the window */
	gtk_widget_show(win_conf);

	return;
}

/* called when playback starts */
static void
ddf_playback_start(void)
{
	printf("DDF: Playback starting\n");
}

/* called when playback stops */
static void
ddf_playback_stop(void)
{
	printf("DDF: Playback stopping\n");
	
	int channel, bucket;
	pthread_mutex_lock(mutex);
	/* reset shared frequency data to zeroes */
	memset(shared_freq_data, 0, sizeof(gint16) * 2 * 256);
	pthread_mutex_unlock(mutex);
}

/* render the frequency data.
 * don't put anything in this function,
 * use worker_func() instead. this must
 * return quickly. */
static void
ddf_render_freq(gint16 freq_data[2][256])
{
	int channel, bucket;
	/* lock shared freq data */
	pthread_mutex_lock(mutex);
	/* copy shared frequency data */
	for(channel = 0; channel < 2; channel++) {
		for(bucket = 0; bucket < 256; bucket++) {
			shared_freq_data[channel][bucket] = freq_data[channel][bucket];
		}
	}
	/* unlock shared freq data */
	pthread_mutex_unlock(mutex);
}
